package internal

import "encoding/json"

type Building struct {
	MaxLevel        int
	MinLevel        int
	Wood            int
	Stone           int
	Iron            int
	Pop             int
	WoodFactor      float64
	StoneFactor     float64
	IronFactor      float64
	PopFactor       float64
	BuildTime       float64
	BuildTimeFactor float64
}

type BuildingInfo struct {
	Main Building
}

type NullBuildingInfo struct {
	Value BuildingInfo
	Valid bool
}

func (b NullBuildingInfo) MarshalJSON() ([]byte, error) {
	if !b.Valid {
		return []byte("null"), nil
	}

	return json.Marshal(b.Value)
}

func (b *NullBuildingInfo) UnmarshalJSON(data []byte) error {
	// Ignore null, like in the main JSON package.
	if string(data) == "null" {
		return nil
	}

	if err := json.Unmarshal(data, &b.Value); err != nil {
		return err
	}

	b.Valid = true

	return nil

}
