-- https://docs.sqlc.dev/en/latest/reference/query-annotations.html
-- https://docs.sqlc.dev/en/latest/howto/named_parameters.html

-- https://docs.sqlc.dev/en/latest/howto/insert.html
-- https://dev.to/forbeslindesay/postgres-unnest-cheat-sheet-for-bulk-operations-1obg
-- name: CreateOrUpdateServers :many
INSERT INTO servers (key, url, open, version_code, config, building_info, unit_info)
VALUES (UNNEST(@key::varchar(100)[]), UNNEST(@url::varchar(255)[]), UNNEST(@open::boolean[]),
        UNNEST(@version_code::varchar(6)[]), UNNEST(@config::jsonb[]), UNNEST(@building_info::jsonb[]),
        UNNEST(@unit_info::jsonb[]))
ON CONFLICT ON CONSTRAINT servers_pkey DO UPDATE SET url  = EXCLUDED.url,
                                                     open = EXCLUDED.open,
                                                     config = EXCLUDED.config,
                                                     building_info = EXCLUDED.building_info,
                                                     unit_info = EXCLUDED.unit_info
RETURNING *;

-- https://docs.sqlc.dev/en/latest/howto/select.html
-- https://docs.sqlc.dev/en/stable/reference/changelog.html#sqlc-embed
-- name: ListServers :many
SELECT sqlc.embed(server), sqlc.embed(version)
from servers server
INNER JOIN public.versions version on version.code = server.version_code
WHERE (server.version_code = ANY (sqlc.narg(version_code)::varchar(6)[]) OR sqlc.narg(version_code) IS NULL)
  AND (server.open = sqlc.narg(open)::boolean OR sqlc.narg(open) IS NULL)
  AND (server.special = sqlc.narg(special)::boolean OR sqlc.narg(special) IS NULL)
  AND (server.key = ANY (sqlc.narg(key)::varchar(100)[]) OR sqlc.narg(key) IS NULL)
ORDER BY CASE WHEN (@sort::text[])[1] = 'KEY_ASC' THEN server.key END ASC,
         CASE WHEN (@sort::text[])[1] = 'KEY_DESC' THEN server.key END DESC,
         CASE WHEN (@sort::text[])[1] = 'OPEN_ASC' THEN server.open END ASC,
         CASE WHEN (@sort::text[])[1] = 'OPEN_DESC' THEN server.open END DESC,
         CASE WHEN (@sort::text[])[2] = 'KEY_ASC' THEN server.key END ASC,
         CASE WHEN (@sort::text[])[2] = 'KEY_DESC' THEN server.key END DESC,
         CASE WHEN (@sort::text[])[2] = 'OPEN_ASC' THEN server.open END ASC,
         CASE WHEN (@sort::text[])[2] = 'OPEN_DESC' THEN server.open END DESC
LIMIT @pag_limit OFFSET @pag_offset;

-- https://docs.sqlc.dev/en/latest/howto/query_count.html
-- name: CountServersByVersionCode :many
SELECT version_code::varchar, cnt::bigint from count_servers_by_version_code() res;

-- https://docs.sqlc.dev/en/latest/howto/update.html
-- name: UpdateServer :one
UPDATE servers
SET
    open = coalesce(sqlc.narg('open'), open)
WHERE key = sqlc.arg('key')
RETURNING *;

-- https://docs.sqlc.dev/en/stable/howto/delete.html
-- name: DeleteServer :one
DELETE FROM servers WHERE key IN (SELECT key FROM servers WHERE player_data_updated_at IS NULL LIMIT 1) RETURNING *;
