package internal

import "encoding/json"

type Unit struct {
	BuildTime      float64
	Pop            int
	Speed          float64
	Attack         int
	Defense        int
	DefenseCavalry int
	DefenseArcher  int
	Carry          int
}

type UnitInfo struct {
	Spear Unit
}

type NullUnitInfo struct {
	Value UnitInfo
	Valid bool
}

func (u NullUnitInfo) MarshalJSON() ([]byte, error) {
	if !u.Valid {
		return []byte("null"), nil
	}

	return json.Marshal(u.Value)
}

func (u *NullUnitInfo) UnmarshalJSON(data []byte) error {
	// Ignore null, like in the main JSON package.
	if string(data) == "null" {
		return nil
	}

	if err := json.Unmarshal(data, &u.Value); err != nil {
		return err
	}

	u.Valid = true

	return nil

}
